/* +-------------+---------+-----------------------+
   | John Breaux | jab0910 | JohnBreaux@my.unt.edu |
   +-------------+---------+-----------------------+
   |              Created  2022-04-17              |
   +-----------------------------------------------+  */
#include <fstream>
#include <regex>
#include "graph.hpp"

#define MAX_INT_LEN 16

// Vector of regices, in order of precedence. Earlier entries will be checked before later entries.
std::vector<std::regex> patterns = {
   //* Rule 1: A line starting with a % is a comment, and should not be parsed
   std::regex("^\\s*(%)"),
   //* Rule 2: A line starting with `num_processes=` contains the number of processes
   std::regex("^.*(num_processes)\\s*=\\s*(\\d+).*"),
   //* Rule 3: A line starting with `num_resources=` contains the number of resources
   std::regex("^.*(num_resources)\\s*=\\s*(\\d+)"),
   //* Rule 4: A line containing only comma-separated values should be captured completely
   std::regex("^([\\-0-9, ]+)") // matches comma-separated space-separated signed decimal integers
};

// convert comma-separated string s to vector<int>
std::vector<int> stovi (const std::string &s);

// TODO: Implement reading from a file
void graph::read(std::string filename) {

   // Open file with name filename as read-only
   std::fstream f; f.open(filename, f.in);
   // TODO: Check for file IO errors (I might have a solution for that in another project)


   while (!f.eof()) {
      // acquire a line
      std::string line;
      std::getline (f, line);

      // Iterate over each pattern, and grab the associated data
      for (auto pattern: patterns) {
         std::smatch res;
         if (std::regex_search (line, res, pattern)) {
            // get the pattern type, value
            std::string type = res.format("$1"), value = res.format("$2");
            // Handle the pattern
            //   Rule 1: If line is a comment, ignore it and move on
            if (type == "%"); else

            //   Rule 2: If num_processes= matched, assign value to num_processes
            if (type == "num_processes") { num_processes = std::stoi(value); } else

            //   Rule 3: If num_resources= matched, assign value to num_resources
            if (type == "num_resources") { num_resources = std::stoi(value); } else

            //   Rule 4: If this line is a comma-separated list of numbers,
            //     and this is the first match, assign it to resource counts
            if (!resource_counts.size()) { resource_counts = stovi(type);    } else
            //     and this is a subsequent match, push it onto the matrix
            { matrix.data.push_back(stovi(type)); }

            // If a pattern is matched, go to the next line and skip evaluating other patterns
            break;
         }
      }
   }

   // Close file
   f.close();
   // TODO: Check for file IO errors (Shouldn't be any)

   return;
}

std::vector<int> stovi (const std::string &s) {
   // Create the number classifier
   std::regex pattern("[0-9]+");
   std::sregex_token_iterator first{s.begin(), s.end(), pattern, 0}, last;
   // Match the numbers
   std::vector<int> res;
   for (auto i = first; i != last; ++i) res.push_back(atoi(i->str().c_str()));
   // give the numbers back
   return res;
}